@@ -1,10 +1,12 @@
+/*!
+ * Copyright 2014 by Contributors
+ * \file xgboost_evaluation-inl.hpp
+ * \brief evaluation metrics for regression and classification and rank
+ * \author Kailong Chen, Tianqi Chen
+ */
 #ifndef XGBOOST_LEARNER_EVALUATION_INL_HPP_
 #define XGBOOST_LEARNER_EVALUATION_INL_HPP_
-/*!
-* \file xgboost_evaluation-inl.hpp
-* \brief evaluation metrics for regression and classification and rank
-* \author Kailong Chen, Tianqi Chen
-*/
+
 #include <vector>
 #include <utility>
 #include <string>
@@ -12,13 +14,14 @@
 #include <climits>
 #include <algorithm>
 #include "../sync/sync.h"
+#include "../utils/math.h"
 #include "./evaluation.h"
 #include "./helper_utils.h"
 
 namespace xgboost {
 namespace learner {
-/*! 
- * \brief base class of elementwise evaluation 
+/*!
+ * \brief base class of elementwise evaluation
  * \tparam Derived the name of subclass
  */
 template<typename Derived>
@@ -27,8 +30,9 @@ struct EvalEWiseBase : public IEvaluator {
                      const MetaInfo &info,
                      bool distributed) const {
     utils::Check(info.labels.size() != 0, "label set cannot be empty");
-    utils::Check(preds.size() % info.labels.size() == 0,
-                 "label and prediction size not match");
+    utils::Check(preds.size() == info.labels.size(),
+                 "label and prediction size not match"\
+                 "hint: use merror or mlogloss for multi-class classification");
 
     const bst_omp_uint ndata = static_cast<bst_omp_uint>(info.labels.size());
 
@@ -45,16 +49,15 @@ struct EvalEWiseBase : public IEvaluator {
     }
     return Derived::GetFinal(dat[0], dat[1]);
   }
-  /*! 
-   * \brief to be implemented by subclass, 
-   *   get evaluation result from one row 
+  /*!
+   * \brief to be implemented by subclass,
+   *   get evaluation result from one row
    * \param label label of current instance
    * \param pred prediction value of current instance
-   * \param weight weight of current instance
    */
   inline static float EvalRow(float label, float pred);
-  /*! 
-   * \brief to be overide by subclas, final trasnformation 
+  /*!
+   * \brief to be overide by subclas, final trasnformation
    * \param esum the sum statistics returned by EvalRow
    * \param wsum sum of weight
    */
@@ -83,7 +86,15 @@ struct EvalLogLoss : public EvalEWiseBase<EvalLogLoss> {
     return "logloss";
   }
   inline static float EvalRow(float y, float py) {
-    return - y * std::log(py) - (1.0f - y) * std::log(1 - py);
+    const float eps = 1e-16f;
+    const float pneg = 1.0f - py;
+    if (py < eps) {
+      return -y * std::log(eps) - (1.0f - y)  * std::log(1.0f - eps);
+    } else if (pneg < eps) {
+      return -y * std::log(1.0f - eps) - (1.0f - y)  * std::log(eps);
+    } else {
+      return -y * std::log(py) - (1.0f - y) * std::log(pneg);
+    }
   }
 };
 
@@ -98,13 +109,107 @@ struct EvalError : public EvalEWiseBase<EvalError> {
   }
 };
 
+/*! \brief loglikelihood of poission distribution */
+struct EvalPoissionNegLogLik : public EvalEWiseBase<EvalPoissionNegLogLik> {
+  virtual const char *Name(void) const {
+    return "poisson-nloglik";
+  }
+  inline static float EvalRow(float y, float py) {
+    const float eps = 1e-16f;
+    if (py < eps) py = eps;
+    return utils::LogGamma(y + 1.0f) + py - std::log(py) * y;
+  }
+};
+
+/*!
+ * \brief base class of multi-class evaluation
+ * \tparam Derived the name of subclass
+ */
+template<typename Derived>
+struct EvalMClassBase : public IEvaluator {
+  virtual float Eval(const std::vector<float> &preds,
+                     const MetaInfo &info,
+                     bool distributed) const {
+    utils::Check(info.labels.size() != 0, "label set cannot be empty");
+    utils::Check(preds.size() % info.labels.size() == 0,
+                 "label and prediction size not match");
+    const size_t nclass = preds.size() / info.labels.size();
+    utils::Check(nclass > 1,
+                 "mlogloss and merror are only used for multi-class classification,"\
+                 " use logloss for binary classification");
+    const bst_omp_uint ndata = static_cast<bst_omp_uint>(info.labels.size());
+    float sum = 0.0, wsum = 0.0;
+    int label_error = 0;
+    #pragma omp parallel for reduction(+: sum, wsum) schedule(static)
+    for (bst_omp_uint i = 0; i < ndata; ++i) {
+      const float wt = info.GetWeight(i);
+      int label =  static_cast<int>(info.labels[i]);
+      if (label >= 0 && label < static_cast<int>(nclass)) {
+        sum += Derived::EvalRow(label,
+                                BeginPtr(preds) + i * nclass,
+                                nclass) * wt;
+        wsum += wt;
+      } else {
+        label_error = label;
+      }
+    }
+    utils::Check(label_error >= 0 && label_error < static_cast<int>(nclass),
+                 "MultiClassEvaluation: label must be in [0, num_class)," \
+                 " num_class=%d but found %d in label",
+                 static_cast<int>(nclass), label_error);
+    float dat[2]; dat[0] = sum, dat[1] = wsum;
+    if (distributed) {
+      rabit::Allreduce<rabit::op::Sum>(dat, 2);
+    }
+    return Derived::GetFinal(dat[0], dat[1]);
+  }
+  /*!
+   * \brief to be implemented by subclass,
+   *   get evaluation result from one row
+   * \param label label of current instance
+   * \param pred prediction value of current instance
+   * \param nclass number of class in the prediction
+   */
+  inline static float EvalRow(int label,
+                              const float *pred,
+                              size_t nclass);
+  /*!
+   * \brief to be overide by subclas, final trasnformation
+   * \param esum the sum statistics returned by EvalRow
+   * \param wsum sum of weight
+   */
+  inline static float GetFinal(float esum, float wsum) {
+    return esum / wsum;
+  }
+  // used to store error message
+  const char *error_msg_;
+};
 /*! \brief match error */
-struct EvalMatchError : public EvalEWiseBase<EvalMatchError> {
+struct EvalMatchError : public EvalMClassBase<EvalMatchError> {
   virtual const char *Name(void) const {
     return "merror";
   }
-  inline static float EvalRow(float label, float pred) {
-    return static_cast<int>(pred) != static_cast<int>(label);
+  inline static float EvalRow(int label,
+                              const float *pred,
+                              size_t nclass) {
+    return FindMaxIndex(pred, nclass) != static_cast<int>(label);
+  }
+};
+/*! \brief match error */
+struct EvalMultiLogLoss : public EvalMClassBase<EvalMultiLogLoss> {
+  virtual const char *Name(void) const {
+    return "mlogloss";
+  }
+  inline static float EvalRow(int label,
+                              const float *pred,
+                              size_t nclass) {
+    const float eps = 1e-16f;
+    size_t k = static_cast<size_t>(label);
+    if (pred[k] > eps) {
+      return -std::log(pred[k]);
+    } else {
+      return -std::log(eps);
+    }
   }
 };
 
@@ -137,7 +242,7 @@ struct EvalCTest: public IEvaluator {
           tpred.push_back(preds[i + (k + 1) * ndata]);
           tinfo.labels.push_back(info.labels[i]);
           tinfo.weights.push_back(info.GetWeight(i));
-        }        
+        }
       }
       wsum += base_->Eval(tpred, tinfo);
     }
@@ -225,7 +330,7 @@ struct EvalPrecisionRatio : public IEvaluator{
                      const MetaInfo &info,
                      bool distributed) const {
     utils::Check(!distributed, "metric %s do not support distributed evaluation", Name());
-    utils::Check(info.labels.size() != 0, "label set cannot be empty");    
+    utils::Check(info.labels.size() != 0, "label set cannot be empty");
     utils::Assert(preds.size() % info.labels.size() == 0,
                   "label size predict size not match");
     std::vector< std::pair<float, unsigned> > rec;
@@ -241,7 +346,8 @@ struct EvalPrecisionRatio : public IEvaluator{
   }
 
  protected:
-  inline double CalcPRatio(const std::vector< std::pair<float, unsigned> >& rec, const MetaInfo &info) const {
+  inline double CalcPRatio(const std::vector< std::pair<float, unsigned> >& rec,
+                           const MetaInfo &info) const {
     size_t cutoff = static_cast<size_t>(ratio_ * rec.size());
     double wt_hit = 0.0, wsum = 0.0, wt_sum = 0.0;
     for (size_t j = 0; j < cutoff; ++j) {
@@ -269,7 +375,7 @@ struct EvalAuc : public IEvaluator {
     utils::Check(info.labels.size() != 0, "label set cannot be empty");
     utils::Check(preds.size() % info.labels.size() == 0,
                  "label size predict size not match");
-    std::vector<unsigned> tgptr(2, 0); 
+    std::vector<unsigned> tgptr(2, 0);
     tgptr[1] = static_cast<unsigned>(info.labels.size());
 
     const std::vector<unsigned> &gptr = info.group_ptr.size() == 0 ? tgptr : info.group_ptr;
@@ -314,8 +420,8 @@ struct EvalAuc : public IEvaluator {
     }
     if (distributed) {
       float dat[2];
-	  dat[0] = static_cast<float>(sum_auc);
-	  dat[1] = static_cast<float>(ngroup);      
+      dat[0] = static_cast<float>(sum_auc);
+      dat[1] = static_cast<float>(ngroup);
       // approximately estimate auc using mean
       rabit::Allreduce<rabit::op::Sum>(dat, 2);
       return dat[0] / dat[1];
@@ -360,8 +466,8 @@ struct EvalRankList : public IEvaluator {
     }
     if (distributed) {
       float dat[2];
-	  dat[0] = static_cast<float>(sum_metric);
-	  dat[1] = static_cast<float>(ngroup);      
+      dat[0] = static_cast<float>(sum_metric);
+      dat[1] = static_cast<float>(ngroup);
       // approximately estimate auc using mean
       rabit::Allreduce<rabit::op::Sum>(dat, 2);
       return dat[0] / dat[1];
@@ -386,7 +492,7 @@ struct EvalRankList : public IEvaluator {
     }
   }
   /*! \return evaluation metric, given the pair_sort record, (pred,label) */
-  virtual float EvalMetric(std::vector< std::pair<float, unsigned> > &pair_sort) const = 0;
+  virtual float EvalMetric(std::vector< std::pair<float, unsigned> > &pair_sort) const = 0; // NOLINT(*)
 
  protected:
   unsigned topn_;
@@ -421,13 +527,13 @@ struct EvalNDCG : public EvalRankList{
     double sumdcg = 0.0;
     for (size_t i = 0; i < rec.size() && i < this->topn_; ++i) {
       const unsigned rel = rec[i].second;
-      if (rel != 0) { 
+      if (rel != 0) {
         sumdcg += ((1 << rel) - 1) / std::log(i + 2.0);
       }
     }
     return static_cast<float>(sumdcg);
   }
-  virtual float EvalMetric(std::vector< std::pair<float, unsigned> > &rec) const {
+  virtual float EvalMetric(std::vector< std::pair<float, unsigned> > &rec) const { // NOLINT(*)
     std::stable_sort(rec.begin(), rec.end(), CmpFirst);
     float dcg = this->CalcDCG(rec);
     std::stable_sort(rec.begin(), rec.end(), CmpSecond);
